

############ R LAB 7th October 2019
############ Sampling Design
########### European Indicators of poverty and vulnerabilities for Sustainable
########### Development Goal and Seminars

# If you have any questions please contact me at: gaia.bertarelli@ec.unipi.it

# Room 238 2^nd floor, DEM.

# Introduction to R and Survey packages (part 2)

# OBJECT AND INDEXING

#x<-matrix(c(1,2,3,4),2,2)   
#class(x)
#x 

#y<-c(5, "Hello!")
#class(y)
#y

#z<-list(a=c(1,2,3),b=matrix(c(1,2,3,4),2,2))
#str(z) # compact and display the structure of an arbitrary R object
#z$b   # $ extracts parts of the objects subset arrays


# Objects live in the Global Enviroment, a workspace that can be browsed, modified and saved for future use. 
# Useful related commands include
# ls() lists the contents of the workspace
# rm() remove an object (useful rm(list=ls())) to erease everything in the workspace
# getwd() setwd() get and see the working directory
# list.files() list files in the working directory
# save.image() save current workspace to a .RData file

# WORKING SPACE, DIRECTORY, I/O

# some useful function to read and write text data files are:
# scan(); read.table(), read.csv()   (to read data)
# write(); wrte.table();sink()  (to write text data file)

# For other data formats specific packages (e.g. foreign and readr). 

# Other useful output function include
# print() cat() paste()
# plot() points () lines() abline()
#jpg() pdf() png() postscript()

# Remember to dev.off() if you output an image(pdf file)

# BASIC STATISTICAL FUNCTION

# originally created by statistician, R encompasses all sort of statistical routines. Some example include:
# summary(), quantile ()
# mean() colMeans() rowMeans()
# var() sd() cov() cor()
# dDISTR() pDISTR() qDISTR() rDISTR()
# lm() glm() anova() prcomp() factanal()
# t.test() var.test() ks.test() wilcox.test() 

# STATISTICAL PLOT
# Beside the generic plot() function, there are many tools to produce pecific plots. Examples include:
# hist() histogram
# bocplot() box and whiskers plot
# density() kernel density argument for plot()
# acf() autocorrelation function
# pairs() pairwise scatterplot
# image() contour() heatmaps and contour plot

rm(list=ls())



###############################################################
##
# SURVEY SAMPLING IN R
##
##############################################################

install.packages(c('sampling','survey'))

library(sampling)

# Methods for sampling: we can find in this packages: 
# (a) function for drawing sampling and calibration
# (b) HT and calibration estimators  (HTestimator and HTstrata)
# (c) implement sampling schemes

library(survey)
# Methods for the analysis of  complex survey data

# These packages encompass function for dealing with the most 
# common sampling design:
# - computation of inclusion probability
# - sample extraction
# - estimation of population quantities

# Some interesting datasets to exercise with are also provide

# Let's begin by understanding how the survey package works. 
# Go to packages, click on survey
# You can find some usefull example in "Vignettes

# To use package survey takes two step

# a) First you have to specify the sampling design of the 
#    survey using "svydesign()" function. 
# b) Then, the particular statistical method or model is calculated 
#    using a special function that calls on the 
#   specified survey design.  
#   These special functions all start with "svy" and include (for example):
# 1. svymean() to compute the mean 
# 2. svytotal() to compute the total

# Let's check the basic syntax of svydesign() function from the help

# More detail about read.dta or any other R function can be viewed by typing a question mark
# in front of the function name in the R console
?svydesign

########

# SIMPLE RANDOM SAMPLING

#######

# Exercise 1

#######
# Simple random sampling (SRS) provides a natural starting point 
# for a discussion of probability sampling methods. 
# It is the simplest method and it underlies many of the more complex methods.
# Simple random sampling is a sampling scheme with the property that any of the
# possible subsets of n distinct elements, from the population of N elements, 
# is equally likely to be the chosen sample.
# Every element in the population has the same probability of being selected for the sample, 
# and the joint probabilities of sets of elements being selected are equal

# Select a sample

set.seed(1985)    # Random Number Generation: fix it to Reproducibility
?srswor   # simple random rampling without replacement
s<-srswor(n=3,N=10) 


# The sample is (I'm not saving anyhing)
(1:10)[s==1]


# Let's try with example data from the package

# In the R language individual data sets are stored as data.frame objects, 
# allowing users to load as many tables into 
# working memory as necessary for the analysis

data(belgianmunicipalities)
?belgianmunicipalities

# This data provides information about the Belgian population of July 1, 2004
# compared to that of July 1, 2003
# and some financial information about the municipality incomes at the end of 2001.

# Try to understand better our data
dim(belgianmunicipalities)

# head() and tail () Returns the first or last parts of a vector, matrix, table, data frame or function.
head(belgianmunicipalities)
tail(belgianmunicipalities)
# 

length(belgianmunicipalities$Commune) #nrow

summary(belgianmunicipalities)
hist(belgianmunicipalities$medianincome)
# 
name=belgianmunicipalities$Commune

# Men04 is the number of men on July 1, 2004

binary<-ifelse(belgianmunicipalities$Men04>3331,1,0)  # ifelse(test, yes, no)(1st Qu.:  3331)
belgianmunicipalities<-cbind(belgianmunicipalities,binary) # add variable binary to our dataset
names(belgianmunicipalities)[18]<-"Bin"                  # give the name "Bin" to binary variable in the dataset

n=50  # sample size  (50/589=8%)
N=589 # population size

# Select sample
s=srswor(n,N)  # vector 589*1
as.vector(name[s==1])  # which communes are sampled (are in the sample)?
id<-(1:N)[s==1]   # or similary which(s==1)     
id   # the position of the sampled commune
sample.srs= belgianmunicipalities[id,]   # select data from the sampled communes. Create a new data.frame (see in Data)

sample.srs<-cbind(sample.srs,N)
head(sample.srs)
sample.srs <-cbind(sample.srs,N/n) # weights from a SRS without replacement
# simple random sampling: all the units have the same probaility of being selected
head(sample.srs)
names(sample.srs)[19]<-"fpc"
names(sample.srs)[20]<-"pw"    


srs_design<-svydesign(id=~1,fpc=~fpc,data=sample.srs)
# id=~1 I do not want any cluster
# fpc: finite population correction (see details in the help)
# This is used when the sampling fraction 
# (the number of elements or respondents sampled relative to the population) becomes large. 
# The FPC is used in the calculation of the standard error of the estimate.  
# If the value of the FPC is close to 1, it will have little impact and can be safely ignored. 

srs_design
summary(srs_design)

# Finite population correction (FPC) assumes simple random sampling without 
# replacement of PSUs within each stratum with no subsampling within PSUs.
# Weights must represent population totals for deff to be correct when
# using an FPC.  Note: deft is invariant to the scale of weights.
# Try to estimate the mean of popuation
mean_inc<-svymean(~ averageincome,srs_design)   # HT
mean_inc


CI_l<-mean_inc[1]-qnorm(0.975)* 511.99  # mean_inc[1] value in the first position of object mean_inc
CI_u<-mean_inc[1]+qnorm(0.975)* 511.99
CI_l
CI_u

confint(mean_inc)

# What is the population mean?
mean(belgianmunicipalities$averageincome)


# Different Approach

# Compute the mean "by hand
# remember that sample.srs is our sample
mean(sample.srs$averageincome)  # mean income in our data 
sqrt(((N-n)/(N))*(var(sample.srs$averageincome)/n))  # standard error in our sample (without replacement from a finite population)
var(sample.srs$averageincome)/mean(sample.srs$averageincome) 

# We can also compute the estimate of total
tot<-svytotal(~Tot04,srs_design)
tot



# Now we start considering the weights

nofpc<-svydesign(id=~1,weights=~pw,data= sample.srs)
summary(nofpc)
  
mean.weight<-svymean(~ averageincome, nofpc)
mean.weight


rm(list=ls())


##############

# ASSIGNMENT 1

# Using Data swissmunicipalities from "sampling" package:
# a) Exploring the data: Let's begin with a bit descriptive of the dataset, tables and plot
# b) Using a SRS (sample units: municipality name. sample: 10% of the population) 
# compute the estimates of the proportion of men and women aged between 20 and 39 and the mean of the total population 
# What you can say about these quantities? Compare the survey and the "by-hand" results.

#############
##################
# ASSIGNMENT 2

# Example taken from Lehtonen and Pahkinen's Pratical Methods for Design and
# Analysis of compex Survey. page 29. Table 2.4. Estimates from a simple random sample drawn 
# without replacement. n=8, N=32

# Import the dataset from text directly into R using the read.table function and the text= parameter
# specifying the entire data set. The syntax n indicates the end of one line of data

province<-read.table(text=
                       "id cluster ue91 lab91
                     1 1 4123 33786
                     2 4 760 5919
                     3 5 721 4930
                     4 15 142 675
                     5 18 187 1448
                     6 26 331 2543
                     7 30 127 1084
                     8 31 219 1330",
                     header=TRUE)

# a) add two columns to the province data.frame object. the columns (variable) are named fpc and weights. What are the
# right value of these variables?
# b) Construct a "survey.design" object called "province.design" specifying a simple random sampling design.
# c) View the weighted total population of this survey design, by referring to the weights column.
# d) Print the mean and the standard errr of the ue91 variable 
# e) Print the weighted total and the standard error of the ue91 variable

rm(list=ls())
#################

# Kruuk et al. (1990) used a stratified random sampling design to estimate the number of otter (Lutra lutra) dens or holts along a 1400km 
# coastline of the Shetland Islands.2 The coastline was divided into 237 5km habitable sections, with each section classified as one of four types: 
# cliffs over 10m (89 sections), agricultural (61 sections), peat (40 sections), and non-peat (47 sections). The sample featured here was collected 
# using stratified random sampling, but for this example it will be treated as if it had been collected using simple random sampling. 
# The data are available in the  SDaA package. 
# If you have not done so already you will need to install this package using install.packages("SDaA"). To access the data load the package.

#install.packages('SDaA')
library(SDaA)
library(survey)
data(otters) # The data frame is called otters.
head(otters)
#Each observation includes the section number, the type of habitat, and the number of holts. Although not necessary for the analysis, 
#it would be helpful to have the stratum names be more descriptive. 
#This can be done by changing habitat into a factor using the factor function and assigning labels to the strata.
otters$habitat <- factor(otters$habitat, labels = c("cliffs", "agricultural", 
                                                    "peat", "non-peat"))
head(otters)

# SPECIFICATION OF THE DESIGN
# Before specifying the design it is necessary to include within the data frame the population size for the finite population correction. 
# This can be done by creating a variable N.
otters$N <- 237
head(otters)
#The svydesign function in the survey package is used to create a survey design object that includes information about the design and the data. 
#A simple random sampling design can be specified as follows.
mydesign <- svydesign(id = ~1, data = otters, fpc = ~N)
# The argument ids = ~1 has to do with if or how elements are grouped within sampling units, such as in cluster sampling. 
# But for simple random sampling simply specify ids = ~1 to indicate one element per sampling unit. 
# The argument data = otters indicates the data frame. The argument fpc = ~N indicates the variable that size of the population from which the unit 
# was sampled. In cases where the finite population correction can be safely ignored such as when 1???n/N???1, or when sampling is with replacement, 
# this argument can be omitted. Notice that the variables are always proceeded by a tilde (~). 
# This is a convention of the survey package but is not typical of other packages in R, so it is easy to forget to use it. 
# The object mydesign now includes the data as well as information about the design.

# Inferences can be made by applying special functions to the object created by svydesign. 
# An estimate for the population mean ?? and its associated standard error can be computed using the svymean function. 
# Recall that the estimator of ?? is simply the sample mean \bar{y}, and its estimated variance is (1-n/N)(s^2/n)
svymean(~holts, design = mydesign)
#Thus we estimate that the mean number of holts per section is about 5.44 holts with a bound on the error of estimation of about 20.6031???1.21 holts. 
#The svymean function does not report the bound on the error of estimation , but it can be computed easily "by hand" 
#if we multiply the standard error by z. 
#The confint function can be applied to the result of svymean to compute the confidence interval.
confint(svymean(~holts, design = mydesign))
#Thus we estimate that the mean number of holts per section is between 4.26 and 6.62 holts.

# INFERENCES FOR DOMAIN MEANS

# Domain means and totals can be computed using the svyby function.

svyby(~holts, by = ~habitat, design = mydesign, FUN = svymean)

svyby(~holts, by = ~habitat, design = mydesign, FUN = svytotal)

# Here the by argument specifies the variable that defines the domains. The FUN argument determines what type of parameters should be estimated.

# The domain mean estimator is a ratio so we can compute the estimates for a specific domain by the following code:
otters$d <- ifelse(otters$habitat == "cliffs", 1, 0)
otters$dholts <- otters$holts * otters$d
mydesign <- svydesign(id = ~1, data = otters, fpc = ~N)
svyratio(numerator = ~dholts, denominator = ~d, design = mydesign)
















